using System;
using System.Collections.Generic;
using System.Text;
using System.Net;
using System.Net.Sockets;

namespace SharpProxyService
{
    public class Mapping
    {
        public Socket LocalSocket = null;
        public IPEndPoint RemoteEP;
        public bool Disposed = false;
        public void Dispose() 
        {
            if (!Disposed) 
            {
                Disposed = true;
                if (LocalSocket == null)
                {

                }
                else 
                {

                    try
                    {
                        LocalSocket.Close();
                    }
                    catch { }
                    try
                    {
                        lock (Exchanging)
                        {
                            foreach (Exchange E in Exchanging)
                            {
                                E.Dispose();
                            }
                            Exchanging.Clear();
                        }
                    }
                    catch { }
                    Exchanging = null;
                }

            }
        
        }
        public List<Exchange> Exchanging = new List<Exchange>();
        public Mapping(IPEndPoint _LocalEP, IPEndPoint _RemoteEP) 
        {
            try
            {
                RemoteEP = _RemoteEP;
                //bind local socket
                LocalSocket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
                LocalSocket.Bind(_LocalEP);
                LocalSocket.Listen(10);
                //begin listening
                LocalSocket.BeginAccept(new AsyncCallback(ClientConnected), null);
            }
            catch (Exception E)
            {
                //System.Windows.Forms.MessageBox.Show(E.ToString());
                Dispose();
            }
            //System.Windows.Forms.MessageBox.Show("Loaded");
        }

        void ClientConnected(IAsyncResult ar) 
        {
            try
            {
                //accept connection
                Socket RemoteClientSocket = LocalSocket.EndAccept(ar);
                //clear prev connections if they were disposed (they sockets has been closed already);
                List<Exchange> ToDispose = new List<Exchange>();
                lock (Exchanging)
                {
                    foreach (Exchange E in Exchanging)
                    {
                        if (E.Disposed)
                        {
                            ToDispose.Add(E);
                        }
                    }
                    foreach (Exchange E in ToDispose)
                    {
                        Exchanging.Remove(E);
                    }
                    ToDispose.Clear();
                    //add new
                    Exchanging.Add(new Exchange(RemoteClientSocket, RemoteEP));


                }
                //begin listening again
                LocalSocket.BeginAccept(new AsyncCallback(ClientConnected), null);
            }
            catch 
            {
                Dispose();
            }
        }
    }
    //
    public class Exchange 
    {
        public Socket RemoteClientSocket;
        byte[] ClientData = new byte[1024];

        Socket RemoteServerSocket = null;
        byte[] ServerData = new byte[1024];


        public bool Disposed = false;
        public void Dispose() 
        {
            if (!Disposed) 
            {
                Disposed = true;
                try
                {
                    RemoteClientSocket.Close();
                }
                catch { }
                try
                {
                    if (RemoteServerSocket != null)
                    {
                        RemoteServerSocket.Close();
                    }
                }
                catch { }
                RemoteClientSocket = null;
                RemoteServerSocket = null;
                ServerData = null;
                ClientData = null;
                
            }
        }
        public Exchange(Socket _RemoteClientSocket, IPEndPoint RemoteServerEP) 
        {
            try
            {
                RemoteClientSocket = _RemoteClientSocket;
                RemoteServerSocket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
                RemoteServerSocket.BeginConnect(RemoteServerEP, new AsyncCallback(OnRemoteServerConnected), null);
            }
            catch 
            {
                Dispose();
            }
        }

        void OnRemoteServerConnected(IAsyncResult ar) 
        {
            try
            {
                RemoteClientSocket.BeginReceive(ClientData, 0, ClientData.Length, SocketFlags.None, new AsyncCallback(OnClientDataArrived), null);
                RemoteServerSocket.BeginReceive(ServerData, 0, ServerData.Length, SocketFlags.None, new AsyncCallback(OnServerDataArrived), null);
            }
            catch 
            {
                Dispose();
            }
        }

        void OnClientDataArrived(IAsyncResult ar) 
        {
            try
            {
                int BytesCount = RemoteClientSocket.EndReceive(ar);
                if (BytesCount > 0)
                {
                    //resend
                    RemoteServerSocket.BeginSend(ClientData, 0, BytesCount, SocketFlags.None, new AsyncCallback(NullAsyncResult), null);
                    //and listen again
                    ClientData = new byte[1024];
                    RemoteClientSocket.BeginReceive(ClientData, 0, ClientData.Length, SocketFlags.None, new AsyncCallback(OnClientDataArrived), null);
                }
                else 
                {
                    //dispose if remote side closed
                    Dispose();
                }
            }
            catch 
            {
                //dispose if any error
                Dispose();
            }
        }

        void OnServerDataArrived(IAsyncResult ar)
        {
            try
            {
                int BytesCount = RemoteServerSocket.EndReceive(ar);
                if (BytesCount > 0)
                {
                    //resend
                    RemoteClientSocket.BeginSend(ServerData, 0, BytesCount, SocketFlags.None, new AsyncCallback(NullAsyncResult), null);
                    //and listen again
                    ServerData = new byte[1024];
                    RemoteServerSocket.BeginReceive(ServerData, 0, ServerData.Length, SocketFlags.None, new AsyncCallback(OnServerDataArrived), null);
                }
                else
                {
                    //dispose if remote side closed
                    Dispose();
                }
            }
            catch
            {
                //dispose if any error
                Dispose();
            }

        }

        void NullAsyncResult(IAsyncResult ar) { }

    
    }
}
